/*
 *
 * Xilinx, Inc.
 * XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS" AS A 
 * COURTESY TO YOU.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
 * ONE POSSIBLE   IMPLEMENTATION OF THIS FEATURE, APPLICATION OR 
 * STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS IMPLEMENTATION 
 * IS FREE FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE RESPONSIBLE 
 * FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE FOR YOUR IMPLEMENTATION
 * XILINX EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH RESPECT TO 
 * THE ADEQUACY OF THE IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO 
 * ANY WARRANTIES OR REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE 
 * FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY 
 * AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/*
 * 
 *
 * This file is a generated sample test application.
 *
 * This application is intended to test and/or illustrate some 
 * functionality of your system.  The contents of this file may
 * vary depending on the IP in your system and may use existing
 * IP driver functions.  These drivers will be generated in your
 * SDK application project when you run the "Generate Libraries" menu item.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include "xaxidma.h"
#include "xparameters.h"
#include "xil_cache.h"
#include "xscugic.h"
#include "xil_exception.h"
#include "scugic_header.h"
#include "xdevcfg.h"
#include "devcfg_header.h"
#include "xdmaps.h"
#include "dmaps_header.h"
#include "axidma_header.h"
#include "xscutimer.h"
#include "scutimer_header.h"
#include "xscuwdt.h"
#include "scuwdt_header.h"

// A function that wraps all AXI DMA initialization related API calls
int init_dma(XAxiDma *axiDmaPtr){
   XAxiDma_Config *CfgPtr;
   int status;
   // Get pointer to DMA configuration
   CfgPtr = XAxiDma_LookupConfig(XPAR_AXIDMA_0_DEVICE_ID);
   if(!CfgPtr){
      print("Error looking for AXI DMA config\n\r");
      return XST_FAILURE;
   }
   // Initialize the DMA handle
   status = XAxiDma_CfgInitialize(axiDmaPtr,CfgPtr);
   if(status != XST_SUCCESS){
      print("Error initializing DMA\n\r");
      return XST_FAILURE;
   }
   //check for scatter gather mode - this example must have simple mode only
   if(XAxiDma_HasSg(axiDmaPtr)){
      print("Error DMA configured in SG mode\n\r");
      return XST_FAILURE;
   }
   //disable the interrupts
   XAxiDma_IntrDisable(axiDmaPtr, XAXIDMA_IRQ_ALL_MASK,XAXIDMA_DEVICE_TO_DMA);
   XAxiDma_IntrDisable(axiDmaPtr, XAXIDMA_IRQ_ALL_MASK,XAXIDMA_DMA_TO_DEVICE);

   return XST_SUCCESS;
}

void TestHMM()
{

	// Declare a XAxiDma object instance
   XAxiDma axiDma;

   // Local variables
   int i, j;
   int status;

	   // Initialize the platform

	   print("---------------------------------------\n\r");
	   print("- HMM test program -\n\r");
	   print("---------------------------------------\n\r");

	   // Initialize the (simple) DMA engine
	   status = init_dma(&axiDma);

	   if (status != XST_SUCCESS) {

		   return;

	   }

	   const int SamplesCount = 10;

	   short realdata[SamplesCount];

	   volatile short outdata[SamplesCount];

	   // Generate a waveform to be input to FFT
	   for (i = 0; i < SamplesCount; i++)
		   realdata[i] = 20 + (rand()/(double)RAND_MAX) * 5;

	   // *IMPORTANT* - flush contents of 'realdata' from data cache to memory
	   // before DMA. Otherwise DMA is likely to get stale or uninitialized data
	   Xil_DCacheFlushRange((unsigned)realdata, SamplesCount * sizeof(short));

	   // DMA enough data to push out first result data set completely
	   status = XAxiDma_SimpleTransfer(&axiDma, (u32)realdata, SamplesCount * sizeof(short), XAXIDMA_DMA_TO_DEVICE);


	   // Do multiple DMA xfers from the HMM core's output stream

	   for (i = 0; i < 8; i++) {

		   // Setup DMA from PL to PS memory using AXI DMA's 'simple' transfer mode
		   status = XAxiDma_SimpleTransfer(&axiDma, (u32)outdata, SamplesCount*sizeof(short), XAXIDMA_DEVICE_TO_DMA);
	      // Poll the AXI DMA core
		   do {

			   status = XAxiDma_Busy(&axiDma, XAXIDMA_DEVICE_TO_DMA);

		   } while(status);

		   // Data cache must be invalidated for 'outdata' buffer after DMA
		   Xil_DCacheInvalidateRange((unsigned)outdata,	SamplesCount * sizeof(short));

		   // DMA another frame of data to PL
		   if (!XAxiDma_Busy(&axiDma, XAXIDMA_DMA_TO_DEVICE))
			   status = XAxiDma_SimpleTransfer(&axiDma, (u32)realdata, SamplesCount * sizeof(short), XAXIDMA_DMA_TO_DEVICE);

		   printf("\n\rFrame received:\n\r");


		   for (j = 0; j < SamplesCount; j++)
		   {
		   	// Convert the fixed point (s.15) values into floating point values

			   float val = ((float)outdata[j])*(2^(-6));

			   printf("Value received: %10.6f", val);
			}

		   printf("End of frame.\n\r");

		   fflush(stdout);

	   }

	   printf("***************\n\r");

	   printf("* End of test *\n\r");

	   printf("***************\n\r\n\r");

	   return;
}

int main() 
{
   static XScuGic intc;
   static XScuTimer ps7_scutimer_0;
   static XScuWdt ps7_scuwdt_0;
   Xil_ICacheEnable();
   Xil_DCacheEnable();
   print("---Entering main---\n\r");
   

	   {
	      int Status;

	      print("\r\n Running ScuGicSelfTestExample() for ps7_scugic_0...\r\n");

	      Status = ScuGicSelfTestExample(XPAR_PS7_SCUGIC_0_DEVICE_ID);

	      if (Status == 0) {
	         print("ScuGicSelfTestExample PASSED\r\n");
	      }
	      else {
	         print("ScuGicSelfTestExample FAILED\r\n");
	      }
	   }

	   {
	       int Status;

	       Status = ScuGicInterruptSetup(&intc, XPAR_PS7_SCUGIC_0_DEVICE_ID);
	       if (Status == 0) {
	          print("ScuGic Interrupt Setup PASSED\r\n");
	       }
	       else {
	         print("ScuGic Interrupt Setup FAILED\r\n");
	      }
	   }
   

   {
      int Status;

      print("\r\n Running DcfgSelfTestExample() for ps7_dev_cfg_0...\r\n");

      Status = DcfgSelfTestExample(XPAR_PS7_DEV_CFG_0_DEVICE_ID);

      if (Status == 0) {
         print("DcfgSelfTestExample PASSED\r\n");
      }
      else {
         print("DcfgSelfTestExample FAILED\r\n");
      }
   }
   

   {
      int Status;
      
      print("\r\n Running XDmaPs_Example_W_Intr() for ps7_dma_s...\r\n");
      
      Status = XDmaPs_Example_W_Intr(&intc,XPAR_PS7_DMA_S_DEVICE_ID);
      
      if (Status == 0) {
         print("XDmaPs_Example_W_Intr PASSED\r\n");
      }
      else {
         print("XDmaPs_Example_W_Intr FAILED\r\n");
      }
   }
   

   {
      int status;


      print("\r\n Running AxiDMASelfTestExample() for axi_dma_0...\r\n");

      status = AxiDMASelfTestExample(XPAR_AXI_DMA_0_DEVICE_ID);

      if (status == 0) {
         print("AxiDMASelfTestExample PASSED\r\n");
      }
      else {
         print("AxiDMASelfTestExample FAILED\r\n");
      }
   }
   

   {
      int Status;

      print("\r\n Running ScuTimerPolledExample() for ps7_scutimer_0...\r\n");

      Status = ScuTimerPolledExample(XPAR_PS7_SCUTIMER_0_DEVICE_ID);

      if (Status == 0) {
         print("ScuTimerPolledExample PASSED\r\n");
      }
      else {
         print("ScuTimerPolledExample FAILED\r\n");
      }
   }
   {
      int Status;

      print("\r\n Running Interrupt Test  for ps7_scutimer_0...\r\n");

      Status = ScuTimerIntrExample(&intc, &ps7_scutimer_0, \
                                 XPAR_PS7_SCUTIMER_0_DEVICE_ID, \
                                 XPAR_PS7_SCUTIMER_0_INTR);

      if (Status == 0) {
         print("ScuTimerIntrExample PASSED\r\n");
      }
      else {
         print("ScuTimerIntrExample FAILED\r\n");
      }

   }
   
   {
      int Status;

      print("\r\n Running Interrupt Test  for ps7_scuwdt_0...\r\n");

      Status = ScuWdtIntrExample(&intc, &ps7_scuwdt_0, \
                                 XPAR_PS7_SCUWDT_0_DEVICE_ID, \
                                 XPAR_PS7_SCUWDT_0_INTR);

      if (Status == 0) {
         print("ScuWdtIntrExample PASSED\r\n");
      }
      else {
         print("ScuWdtIntrExample FAILED\r\n");
      }

   }

   TestHMM();

   print("---Exiting main---\n\r");
   Xil_DCacheDisable();
   Xil_ICacheDisable();
   return 0;
}
